/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package pe.gob.sunat.tecnologia.receptor.cliente.forms;

import pe.gob.sunat.tecnologia.receptor.cliente.forms.enumeration.Operacion;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Calendar;
import javax.activation.DataHandler;
import javax.swing.JProgressBar;
import javax.swing.JTable;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import pe.gob.sunat.sigad.services.AcuseRecibo;
import pe.gob.sunat.tecnologia.receptor.cliente.service.ClienteEnvioSeidaService;
import pe.gob.sunat.tecnologia.receptor.cliente.service.ClienteEnvioSeidaServiceImpl;

/**
 * <p>Clase encargada de realizar la petición al SEIDA, ya sea para enviar un archivo con
 * el documento aduanero, o para enviar un ticket de consulta para ver el resultado
 * del envio.</p>
 * <p>El envio genera un acuse de recibo: nsigad_acuse_AAAA_9999, donde AAAA es el año de envio y
 * 9999 es el número de ticket.</p>
 * <p>La consulta genera un archivo de respuesta: nsigad_response_AAAAXXXXXXXXXXXXX9999, donde
 * AAAA es el año de envio, XXXX es un timestamp y  9999 es el número de ticket.</p>
 * 
 * @author JVALDEZ
 */
public class Sender extends Thread {
    protected final Log log = LogFactory.getLog(getClass());
    /**
     * ID del hilo
     */
    private String id;
    /**
     * Propiedades de seguridad
     */
    private String ruc;
    private String codigoEnvio;
    private String usuario;
    private String clave;
    /**
     * Ubicacion de loa archivos de salida generados..
     */
    private String rutaArchivos = "./";
    /*
     * propiedades para el envio de los documentos aduaneros
     */
    private String numeroTransaccion;
    private String archivoEnviar;
    /**
     * propiedades para la consulta del resultado del envio
     */
    private String numeroTicket;
    private String anhoEnvio;

    /**
     * parametros de la pc
     */
    private String direccionIP;
    private String macAddress;
    /**
     * parametros de mensajes en la consola
     */
    private javax.swing.JTextArea areaMensajes;
    private JProgressBar progressBar;
    /**
     * Datos de la operacion
     */
    private Operacion operacion;

    /**
     * Lista de los envios a consultar, utilizada para setear el archivo de respiesta generado.
     */
    private JTable listaElementos;

    public Sender(String name, Operacion operacion) {
        this.id = name;
        this.operacion = operacion;
    }

    public void run(){
        switch(this.operacion){
            case OP_ENVIAR:
                invocar();break;
            case OP_CONSULTAR:
                consultar();
        }
    }

    /**
     *
     */
    private void invocar(){
        try {

            log.info("invocando...");
            ClienteEnvioSeidaService service = new ClienteEnvioSeidaServiceImpl();
            AcuseRecibo realizarEnvio = service.realizarEnvio(this.numeroTransaccion, this.archivoEnviar, this.ruc + this.usuario + "|" + this.macAddress + "|" + this.direccionIP + "|2", this.clave);
            if (realizarEnvio != null) {
                this.anhoEnvio = realizarEnvio.getAnhoEnvio();
                this.numeroTicket = realizarEnvio.getTicketEnvio();

                File f = new Util().acuseToXML(realizarEnvio, this.rutaArchivos);

                log.info("Envio " + this.id + "("+this.archivoEnviar+") ticket: " + realizarEnvio.getTicketEnvio() + " acuse:" + f.getAbsolutePath());
                areaMensajes.append("Envio " + this.id + "("+this.archivoEnviar+") ticket: " + realizarEnvio.getTicketEnvio() + " acuse:" + f.getAbsolutePath() + "\n");
            }
        } catch (Exception ex) {
            log.error("Envio " + this.id + "("+this.archivoEnviar+"): ha ocurrido un problema al intentar enviar este archivo, y no ha generado ticket ni acuse. Por favor intente nuevamente.");
            log.error(this, ex);
        } finally {
            progressBar.setValue( progressBar.getValue()+1 );


        }

    }
    /**
     *
     */
    private void consultar() {
        FileOutputStream fos = null;
        try {

            String sufijoFileName = this.anhoEnvio + Calendar.getInstance().getTimeInMillis() + this.numeroTicket.trim();

            File f = new File(this.rutaArchivos + "/" + "nsigad_response_"+ sufijoFileName + ".xml");

            fos = new FileOutputStream(f);

            log.info("invocando...");

            String query = "<consulta><tipo>1</tipo><parametros><numeroTicket>"+this.numeroTicket.trim()+"</numeroTicket><annoTicket>"+this.anhoEnvio+"</annoTicket></parametros></consulta>";

            ClienteEnvioSeidaService service = new ClienteEnvioSeidaServiceImpl();
            DataHandler realizarConsulta = service.realizarConsulta(query, this.ruc + this.usuario + "|" + this.macAddress + "|" + this.direccionIP + "|2", this.clave);
            if(realizarConsulta != null){

                realizarConsulta.writeTo(fos);

                log.info("Consulta " + this.id + "("+this.numeroTicket.trim()+","+this.anhoEnvio+") resultado: creado=" + f.exists() + " tamaño="+f.length());
                areaMensajes.append("Consulta " + this.id + "("+this.numeroTicket.trim()+","+this.anhoEnvio+") resultado: creado=" + f.exists() + " tamaño="+f.length() + "\n");

                listaElementos.setValueAt(this.rutaArchivos + "/" + "nsigad_response_"+ sufijoFileName + ".xml", new Integer(this.id), 2);
            }
        } catch (Exception ex) {
            log.error("Consulta " + this.id + "("+this.numeroTicket.trim()+","+this.anhoEnvio+") resultado: Ha ocurrido un problema y no se ha podido crear el archivo de respuesta. Por favor intenete nuevamente.");
            log.error(this, ex);
        } finally {
            progressBar.setValue( progressBar.getValue()+1 );

            try {
                fos.close();
            } catch (IOException ex) {
                log.error(this, ex);
            }
        }
    }

    /**
     * Indica el numero de RUC a utilizar para la autenticacion
     *
     * @param ruc the ruc to set
     */
    public void setRuc(String ruc) {
        this.ruc = ruc;
    }

    /**
     * Indica el Codigo de Envio a utilizar para la autenticacion
     *
     * @param codigoEnvio the codigoEnvio to set
     */
    public void setCodigoEnvio(String codigoEnvio) {
        this.codigoEnvio = codigoEnvio;
    }

    /**
     * Indica el usuario a utilizar para la autenticacion
     *
     * @param usuario the usuario to set
     */
    public void setUsuario(String usuario) {
        this.usuario = usuario;
    }

    /**
     * Indica la clave a utilizar para la autenticacion
     *
     * @param clave the clave to set
     */
    public void setClave(String clave) {
        this.clave = clave;
    }

    /**
     * Indica el numero de transaccion asociada al archivo a procesar
     *
     * @param numeroTransaccion the numeroTransaccion to set
     */
    public void setNumeroTransaccion(String numeroTransaccion) {
        this.numeroTransaccion = numeroTransaccion;
    }

    /**
     * Nombre del archivo a enviar, incluyendo la ruta absoluta.
     *
     * @return the archivoEnviar
     */
    public String getArchivoEnviar() {
        return archivoEnviar;
    }

    /**
     * Indica el nombre del archivo a enviar, incluyendo la ruta absoluta.
     * @param archivoEnviar the archivoEnviar to set
     */
    public void setArchivoEnviar(String archivoEnviar) {
        this.archivoEnviar = archivoEnviar;
    }

    /**
     * Ruta donde se crearan los archivos resultantes, acuses, respuestas de consulta,
     * tickets generados.
     *
     * @return the rutaArchivos
     */
    public String getRutaArchivos() {
        return rutaArchivos;
    }

    /**
     * Indica la ruta donde se crearan los archivos resultantes, acuses, respuestas de consulta,
     * tickets generados.
     *
     * @param rutaArchivos the rutaArchivos to set
     */
    public void setRutaArchivos(String rutaArchivos) {
        this.rutaArchivos = rutaArchivos;
    }

    /**
     * Numero de ticket retornado por el servicio de recepcion del SEIDA.
     *
     * @return the numeroTicket
     */
    public String getNumeroTicket() {
        return numeroTicket;
    }

    /**
     * Indica el numero de ticket del envio.
     *
     * @param numeroTicket the numeroTicket to set
     */
    public void setNumeroTicket(String numeroTicket) {
        this.numeroTicket = numeroTicket;
    }

    /**
     * Indica la operacion a realizar.
     *
     * @param operacion the operacion to set
     * @see com.forms.enumeration.Operacion
     */
    public void setOperacion(Operacion operacion) {
        this.operacion = operacion;
    }

    /**
     * Año del envio
     *
     * @return the anhoEnvio
     */
    public String getAnhoEnvio() {
        return anhoEnvio;
    }

    /**
     * Indica el año del envio
     *
     * @param anhoEnvio the anhoEnvio to set
     */
    public void setAnhoEnvio(String anhoEnvio) {
        this.anhoEnvio = anhoEnvio;
    }

    /**
     * Area donde se muestran las operaciones realizadas por la aplicacion, al
     * momento de realizar envios o consultas.
     *
     * @param areaMensajes the areaMensajes to set
     */
    public void setAreaMensajes(javax.swing.JTextArea areaMensajes) {
        this.areaMensajes = areaMensajes;
    }

    /**
     * Progreso
     *
     * @param progressBar the progressBar to set
     */
    public void setProgressBar(JProgressBar progressBar) {
        this.progressBar = progressBar;
    }

    /**
     * Tabla donde se muestran los tickets a consultar, cada respuesta setea el 
     * nombre del archivo de respuesta resultante en el ticket que corresponda.
     *
     * @return the listaElementos
     */
    public JTable getListaElementos() {
        return listaElementos;
    }

    /**
     * Indica la Tabla donde se muestran los tickets a consultar, cada respuesta setea el
     * nombre del archivo de respuesta resultante en el ticket que corresponda.
     *
     * @param listaElementos the listaElementos to set
     */
    public void setListaElementos(JTable listaElementos) {
        this.listaElementos = listaElementos;
    }

    /**
     * Direccion IP de la estacion cliente, desde donde se esta enviando los
     * archivos o haciendo consultas.
     *
     * @param direccionIP the direccionIP to set
     * @see InetAddress
     */
    public void setDireccionIP(String direccionIP) {
        this.direccionIP = direccionIP;
    }

    /**
     * MAC Address de la estacion cliente, desde donde se esta enviando los
     * archivos o haciendo consultas.
     * 
     * @param macAddress the macAddress to set
     * @see NetworkInterface
     */
    public void setMacAddress(String macAddress) {
        this.macAddress = macAddress;
    }

}
