package pe.gob.sunat.tecnologia.receptor.cliente.forms;

import freemarker.template.Configuration;
import freemarker.template.DefaultObjectWrapper;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import java.util.*;
import java.text.*;
import java.io.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import pe.gob.sunat.sigad.services.AcuseRecibo;

/**
 * Metodos de uso comun de la aplicacion.
 *
 * @author JVALDEZ
 * @see freemarker.template.Template
 * @see freemarker.template.Configuration
 */
public class Util {
    private static Template plantilla = null;
    static {
        /**
         * La carga estatica aqui, permite que la plantilla se compile una sola vez.
         */
        Configuration cfg = new Configuration();
        cfg.setClassForTemplateLoading(pe.gob.sunat.tecnologia.receptor.cliente.forms.Util.class, "/");
        cfg.setObjectWrapper(new DefaultObjectWrapper());
        try {
            // Obtenemos la plantilla
            plantilla = cfg.getTemplate("pe/gob/sunat/tecnologia/receptor/cliente/forms/resources/nsigad_acuse.ftl");
        } catch (IOException ex) {
            Logger.getLogger(Util.class.getName()).log(Level.SEVERE, null, ex);
        }

    }

    /**
     * Obtiene el CanonicalPath del archivo indicado en fileName
     *
     * @param fileName String
     * @return File
     * @see java.io.File
     */
    public static File currentDirForDialog(String fileName) {
        File f = null;
        try {
            // Create a File object containing the canonical path of the
            // desired directory
            f = new File(new File(fileName).getCanonicalPath());
        } catch (IOException e) {
        }
        return f;
    }

    /**
     * Obtiene la hora actual formateada como hh:mm:ss SSS
     * @return String
     */
    public static String getTimeNow() {
        String s;
        Format formatter;
        Date date = new Date();

        formatter = new SimpleDateFormat("hh:mm:ss SSS");
        s = formatter.format(date);
        return s;
    }

    /**
     * Convierte un arreglo de bytes a una cadena en valor hexadecimal.
     *
     * @param bytes byte[]
     * @return String
     * @see java.util.Formatter
     */
    public static String bytesToHexString(byte[] bytes) {
        StringBuilder sb = new StringBuilder(bytes.length * 2);

        Formatter formatter = new Formatter(sb);
        for (byte b : bytes) {
            formatter.format("%02x ", b);
        }
        return sb.toString().trim().replace(' ', ':');
    }

    /**
     * Contiene el objeto acuse a XML, y lo graba en la ruta dirRpta, utiliza
     * freemaker para generar el archivo.
     *
     * @param acuse AcuseRecibo
     * @param dirRpta String
     * @return File
     * @throws Exception
     * @see java.io.FileOutputStream
     */
    public File acuseToXML(AcuseRecibo acuse, String dirRpta) throws Exception{
        File archivoResultado = null;
        FileOutputStream out = null;
        try {
            StringWriter procesador = new StringWriter();
            plantilla.process(acuse, procesador);
            String resultado = procesador.toString();

            archivoResultado = new File(dirRpta + "/nsigad_acuse_" + acuse.getAnhoEnvio() + "_" + acuse.getTicketEnvio() + ".xml");
            out = new FileOutputStream(archivoResultado);
            out.write(resultado.getBytes());

        } catch (TemplateException ex) {
            throw new Exception(ex);
        } finally {
            out.close();
        }
        return archivoResultado;
    }

}
