package pe.gob.sunat.tecnologia.receptor.cliente.service;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.xml.soap.DetailEntry;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.Holder;
import javax.xml.ws.handler.Handler;
import javax.xml.ws.soap.SOAPFaultException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import pe.gob.sunat.sigad.services.AcuseRecibo;
import pe.gob.sunat.sigad.services.RespuestaConsultaBean;

import pe.gob.sunat.tecnologia.receptor.cliente.bean.SeguridadCliente;
import pe.gob.sunat.tecnologia.receptor.cliente.util.holder.SeguridadClienteHolder;
import pe.gob.sunat.tecnologia.receptor.cliente.util.security.handler.ClientSecurityHandler;
import pe.gob.sunat.tecnologia.receptor.service.ReceptorServiceHtm;
import pe.gob.sunat.tecnologia.receptor.service.ReceptorServiceHtm_Service;

public class ClienteEnvioSeidaServiceImpl implements ClienteEnvioSeidaService {
    protected final Log log = LogFactory.getLog(getClass());

    public AcuseRecibo realizarEnvio(String numeroTransaccion, String rutaArchivo, String user, String password) throws Exception {
        AcuseRecibo acuse = null;
        
        FileDataSource fileDataSource = new FileDataSource(rutaArchivo);
        if (!fileDataSource.getFile().exists() || !fileDataSource.getFile().canRead() || fileDataSource.getFile().length() == 0){
            log.error("No se ha podido enviar el archivo " + rutaArchivo + " porque no existe, no tiene permisos o su tamaño es de 0 bytes");
            return acuse;
        }
        // Realizamos la invocaci�n de servicios 
        ReceptorServiceHtm_Service service = new ReceptorServiceHtm_Service();
        
        ReceptorServiceHtm port = service.getReceptorWebServiceServiceImplPort();
        
        // Guardamos la informaci�n de autenticaci�n
        SeguridadCliente cliente = new SeguridadCliente();
        cliente.setUsuario(user);
        cliente.setClave(password);

        SeguridadClienteHolder.set(cliente);
        
        DataHandler dh = new DataHandler(fileDataSource);
            
        List<Handler> chain = new ArrayList<Handler>();
        chain.add(new ClientSecurityHandler());
        ((BindingProvider)port).getBinding().setHandlerChain(chain);
        
        // Realizamos el envio, el resultado del envio ser� almacenado en la variable "acuse"
        try {
            acuse = port.recibirArchivo(numeroTransaccion,dh);
        } catch(SOAPFaultException ex){
            log.error(this, ex);
        }

        // Limpiamos el Holder
        SeguridadClienteHolder.reset();
        
        return acuse;
    }
    
    public DataHandler realizarConsulta(String parametrosConsulta, String user, String password)throws Exception {
        
        DataHandler resultadoConsulta = null;
        
        // Realizamos la invocaci�n de servicios 
        ReceptorServiceHtm_Service service = new ReceptorServiceHtm_Service();
        
        ReceptorServiceHtm port = service.getReceptorWebServiceServiceImplPort();
        
        // Guardamos la informaci�n de autenticaci�n
        SeguridadCliente cliente = new SeguridadCliente();
        cliente.setUsuario(user);
        cliente.setClave(password);
        
        SeguridadClienteHolder.set(cliente);
        
        List<Handler> chain = new ArrayList<Handler>();
        chain.add(new ClientSecurityHandler());
        ((BindingProvider)port).getBinding().setHandlerChain(chain);
        
        Holder holderConsulta = new Holder();
        holderConsulta.value = resultadoConsulta;
        
        RespuestaConsultaBean respuestaConsulta = new RespuestaConsultaBean();
        Holder holderRespuesta = new Holder();
        holderRespuesta.value = respuestaConsulta;
        
        // Realizamos la consulta
        port.realizarConsulta(parametrosConsulta, holderRespuesta, holderConsulta);
        
        resultadoConsulta = (DataHandler) holderConsulta.value;
        
        SeguridadClienteHolder.reset();
        
        return resultadoConsulta;
    }
    
}
