package pe.gob.sunat.tecnologia.receptor.cliente.util.holder;

import pe.gob.sunat.tecnologia.receptor.cliente.bean.SeguridadCliente;

/**
 * Clase que permite trasladar la informacin de autenticacin.
 * 
 * @author Erick Osctegui
 * @version 1.0 
 */

public class SeguridadClienteHolder {

  private static ThreadLocal<SeguridadCliente> tLocal = new ThreadLocal<SeguridadCliente>();
  
  /**
   * Carga la informacin al ThreadLocal.
   * 
   * @param SeguridadCliente seguridadCliente Informacin de cliente.
   * @return void
   */
  
  public static void set(SeguridadCliente seguridadCliente) {
      
      // Obtenemos el objeto
      SeguridadCliente cliente = tLocal.get();

      // Si aun no existe la lista, la generamos
      if (cliente == null) {
          cliente = seguridadCliente;
      } 

      // Lo colocamos en el ThreadLocal
      tLocal.set(cliente);
      
  }
  
  /**
   * Obtiene la informacin del cliente almacenada en el ThreadLocal.
   * 
   * @return SeguridadCliente Informacin de cliente.
   */

  public static SeguridadCliente get() {
      return tLocal.get();
  }
  
  /**
   * Este metodo se encarga de llamar al metodo remove del ThreadLocal, para borrar la lista de log
   * que se ha cargado. Debe ser invocado siempre, cuando se termina de usar la lista. 
   * 
   * @return void
   */
  
  public static void reset(){
      // tLocal.set(null);
      tLocal.remove();
  }
  
}
