package pe.gob.sunat.tecnologia.receptor.cliente.util.security.handler;

import java.util.Set;
import java.util.HashSet;
import javax.xml.namespace.QName;
import javax.xml.soap.SOAPMessage;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;

import com.sun.xml.wss.ProcessingContext;
import com.sun.xml.wss.XWSSProcessorFactory;
import com.sun.xml.wss.XWSSProcessor;
import com.sun.xml.wss.XWSSecurityException;
import java.io.InputStream;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class ClientSecurityHandler implements SOAPHandler<SOAPMessageContext> {
    protected final Log log = LogFactory.getLog(getClass());
   
    XWSSProcessor sprocessor = null;
    XWSSProcessor cprocessor = null;
    /** Creates a new instance of ClientSecurityHandler */
   
    public ClientSecurityHandler() {
        InputStream clientConfig = null;
        try {
            //read client side security config
            //clientConfig = new java.io.FileInputStream(new java.io.File("META-INF/user-pass-authenticate-client.xml"));
            // clientConfig = new java.io.FileInputStream(new File("user-pass-authenticate-client.xml"));

            clientConfig = ClassLoader.getSystemResourceAsStream("pe/gob/sunat/tecnologia/receptor/cliente/forms/resources/user-pass-authenticate-client.xml");

            // clientConfig = new java.io.FileInputStream(new File("user-pass-authenticate-client.xml"));
            //Create a XWSSProcessFactory.
            XWSSProcessorFactory factory = XWSSProcessorFactory.newInstance();
            cprocessor = factory.createProcessorForSecurityConfiguration(clientConfig, new ClientSecurityEnvironmentHandler());
            clientConfig.close();
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        }

    }
    
    public Set<QName> getHeaders() {
         QName securityHeader = new QName("http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd", "Security", "wsse");
         HashSet<QName> headers = new HashSet<QName>();
         headers.add(securityHeader);
         return headers;
    } 

    public boolean handleFault(SOAPMessageContext messageContext) {
        return true;
    }

     public boolean handleMessage(SOAPMessageContext messageContext) {
    	 secureClient(messageContext);
    	 return true;
    }
     
    public void close(MessageContext messageContext) {}
   
    private SOAPMessage createFaultResponse(XWSSecurityException ex) {
        // TODO: add code here
        return null;
    }
    
    private void secureClient(SOAPMessageContext messageContext) {
    	
        Boolean outMessageIndicator = (Boolean) messageContext.get (MessageContext.MESSAGE_OUTBOUND_PROPERTY);
        SOAPMessage message = messageContext.getMessage();
        
        
        if (outMessageIndicator.booleanValue()) {
            log.info("Outbound SOAP secureClient:");
            ProcessingContext context;
            try {
                context = cprocessor.createProcessingContext(message);
                context.setSOAPMessage(message);
                SOAPMessage secureMsg = cprocessor.secureOutboundMessage(context);
                // secureMsg.writeTo(System.out);
                messageContext.setMessage(secureMsg);
            } catch (XWSSecurityException ex) {
                ex.printStackTrace();
                throw new RuntimeException(ex);
            } catch (Exception e) {
                e.printStackTrace();
                throw new RuntimeException(e);
            } 
            return;
        } else {
            log.info("Inbound SOAP secureClient:");
            return;
        }     
    }
}